@@ -1,4 +1,4 @@
-/*	$Id: lowdown.h,v 1.48 2018/12/23 18:52:19 kristaps Exp $ */
+/*	$Id: lowdown.h,v 1.69 2020/01/28 11:19:36 kristaps Exp $ */
 /*
  * Copyright (c) 2017 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -44,7 +44,9 @@ enum	lowdown_type {
 	LOWDOWN_HTML,
 	LOWDOWN_MAN,
 	LOWDOWN_NROFF,
-	LOWDOWN_TREE
+	LOWDOWN_TERM,
+	LOWDOWN_TREE,
+	LOWDOWN_NULL
 };
 
 enum	lowdown_err {
@@ -55,8 +57,6 @@ enum	lowdown_err {
 	LOWDOWN_ERR__MAX
 };
 
-typedef	void (*lowdown_msg)(enum lowdown_err, void *, const char *);
-
 /*
  * All types of Markdown nodes that lowdown understands.
  */
@@ -94,6 +94,7 @@ enum	lowdown_rndrt {
 	LOWDOWN_ENTITY,
 	LOWDOWN_NORMAL_TEXT,
 	LOWDOWN_DOC_HEADER,
+	LOWDOWN_META,
 	LOWDOWN_DOC_FOOTER,
 	LOWDOWN__MAX
 };
@@ -106,8 +107,6 @@ typedef struct hbuf {
 	int 		 buffer_free; /* obj should be freed */
 } hbuf;
 
-/*
- */
 TAILQ_HEAD(lowdown_nodeq, lowdown_node);
 
 enum 	htbl_flags {
@@ -136,8 +135,11 @@ enum	hlist_fl {
 struct	lowdown_meta {
 	char		*key;
 	char		*value;
+	TAILQ_ENTRY(lowdown_meta) entries;
 };
 
+TAILQ_HEAD(lowdown_metaq, lowdown_meta);
+
 enum	lowdown_chng {
 	LOWDOWN_CHNG_NONE = 0,
 	LOWDOWN_CHNG_INSERT,
@@ -153,16 +155,21 @@ struct	lowdown_node {
 	enum lowdown_chng	 chng; /* change type */
 	size_t			 id; /* unique identifier */
 	union {
-		struct rndr_doc_header {
-			struct lowdown_meta *m; /* unescaped */
-			size_t msz;
-		} rndr_doc_header;
+		struct rndr_meta {
+			hbuf key;
+		} rndr_meta;
 		struct rndr_list {
 			enum hlist_fl flags; /* only HLIST_FL_ORDERED */
+			/*
+			 * This is string of size >0 iff
+			 * HLIST_FL_ORDERED and we're parsing
+			 * CommonMark, else it's an empty string.
+			 */
+			char start[10];
 		} rndr_list; 
 		struct rndr_listitem {
 			enum hlist_fl flags; /* all possible flags */
-			size_t num; /* index in ordered */
+			uint32_t num; /* index in ordered */
 		} rndr_listitem; 
 		struct rndr_header {
 			size_t level; /* hN level */
@@ -216,7 +223,7 @@ struct	lowdown_node {
 		} rndr_image;
 		struct rndr_math {
 			hbuf text; /* equation (opaque) */
-			int displaymode;
+			int blockmode;
 		} rndr_math;
 		struct rndr_blockhtml {
 			hbuf text;
@@ -231,9 +238,11 @@ struct	lowdown_node {
  * These options contain everything needed to parse and render content.
  */
 struct	lowdown_opts {
-	lowdown_msg		 msg;
 	enum lowdown_type	 type;
-	void			*arg;
+	size_t			 maxdepth; /* max parse tree depth */
+	size_t			 cols; /* -Tterm width */
+	size_t			 hmargin; /* -Tterm left margin */
+	size_t			 vmargin; /* -Tterm top/bot margin */
 	unsigned int		 feat;
 #define LOWDOWN_TABLES		 0x01
 #define LOWDOWN_FENCED		 0x02
@@ -251,16 +260,19 @@ struct	lowdown_opts {
 #define	LOWDOWN_METADATA	 0x4000
 #define	LOWDOWN_COMMONMARK	 0x8000
 	unsigned int		 oflags;
-#define LOWDOWN_HTML_SKIP_HTML	 0x01
-#define LOWDOWN_HTML_ESCAPE	 0x02
-#define LOWDOWN_HTML_HARD_WRAP	 0x04
-#define LOWDOWN_NROFF_SKIP_HTML	 0x08
-#define LOWDOWN_NROFF_HARD_WRAP	 0x10
-#define LOWDOWN_NROFF_GROFF	 0x20
-#define	LOWDOWN_SMARTY	  	 0x40
-#define LOWDOWN_NROFF_NUMBERED	 0x80
-#define LOWDOWN_HTML_HEAD_IDS	 0x100
-#define LOWDOWN_STANDALONE	 0x200
+#define LOWDOWN_HTML_SKIP_HTML	 0x01 /* skip all HTML */
+#define LOWDOWN_HTML_ESCAPE	 0x02 /* escape HTML (if not skip) */
+#define LOWDOWN_HTML_HARD_WRAP	 0x04 /* paragraph line breaks */
+#define LOWDOWN_NROFF_SKIP_HTML	 0x08 /* skip all HTML */
+#define LOWDOWN_NROFF_HARD_WRAP	 0x10 /* paragraph line breaks */
+#define LOWDOWN_NROFF_GROFF	 0x20 /* use groff extensions */
+#define	LOWDOWN_SMARTY	  	 0x40 /* smart typography */
+#define LOWDOWN_NROFF_NUMBERED	 0x80 /* numbered section headers */
+#define LOWDOWN_HTML_HEAD_IDS	 0x100 /* <hN id="the_name"> */
+#define LOWDOWN_STANDALONE	 0x200 /* emit complete document */
+#define LOWDOWN_TERM_SHORTLINK	 0x400 /* shorten URLs */
+#define	LOWDOWN_HTML_OWASP	 0x800 /* use OWASP escaping */
+#define	LOWDOWN_HTML_NUM_ENT	 0x1000 /* use &#nn; if possible */
 };
 
 struct hdoc;
@@ -269,8 +281,6 @@ typedef struct hdoc hdoc;
 
 __BEGIN_DECLS
 
-const char *lowdown_errstr(enum lowdown_err);
-
 /*
  * High-level functions.
  * These use the "lowdown_opts" to determine how to parse and render
@@ -278,19 +288,14 @@ const char *lowdown_errstr(enum lowdown_err);
  */
 void	 lowdown_buf(const struct lowdown_opts *, 
 		const char *, size_t,
-		char **, size_t *,
-		struct lowdown_meta **, size_t *);
+		char **, size_t *, struct lowdown_metaq *);
 void	 lowdown_buf_diff(const struct lowdown_opts *, 
-		const char *, size_t,
-		const struct lowdown_opts *,
-		const char *, size_t,
-		char **, size_t *);
+		const char *, size_t, const char *, size_t,
+		char **, size_t *, struct lowdown_metaq *);
 int	 lowdown_file(const struct lowdown_opts *, 
-		FILE *, char **, size_t *,
-		struct lowdown_meta **, size_t *);
+		FILE *, char **, size_t *, struct lowdown_metaq *);
 int	 lowdown_file_diff(const struct lowdown_opts *, FILE *, 
-		const struct lowdown_opts *, FILE *,
-		char **, size_t *);
+		FILE *, char **, size_t *, struct lowdown_metaq *);
 
 /* 
  * Low-level functions.
@@ -300,34 +305,34 @@ int	 lowdown_file_diff(const struct lowdown_opts *, FILE *,
 
 hdoc 	*lowdown_doc_new(const struct lowdown_opts *);
 struct lowdown_node
-	*lowdown_doc_parse(hdoc *, const char *, size_t, 
-		struct lowdown_meta **, size_t *);
+	*lowdown_doc_parse(hdoc *, size_t *, const char *, size_t);
 struct lowdown_node
 	*lowdown_diff(const struct lowdown_node *,
-		const struct lowdown_node *);
+		const struct lowdown_node *, size_t *);
 void	 lowdown_doc_free(hdoc *);
+void	 lowdown_metaq_free(struct lowdown_metaq *);
 
 void 	 lowdown_node_free(struct lowdown_node *);
 
 void	 lowdown_html_free(void *);
 void	*lowdown_html_new(const struct lowdown_opts *);
-void 	 lowdown_html_rndr(hbuf *, void *, 
-		struct lowdown_node *);
+void 	 lowdown_html_rndr(hbuf *, struct lowdown_metaq *,
+		void *, const struct lowdown_node *);
+
+void	 lowdown_term_free(void *);
+void	*lowdown_term_new(const struct lowdown_opts *);
+void 	 lowdown_term_rndr(hbuf *, struct lowdown_metaq *,
+		void *, const struct lowdown_node *);
 
 void	 lowdown_nroff_free(void *);
 void	*lowdown_nroff_new(const struct lowdown_opts *);
-void 	 lowdown_nroff_rndr(hbuf *, void *, 
-		struct lowdown_node *);
+void 	 lowdown_nroff_rndr(hbuf *, struct lowdown_metaq *,
+		void *, struct lowdown_node *);
 
 void	 lowdown_tree_free(void *);
 void	*lowdown_tree_new(void);
-void 	 lowdown_tree_rndr(hbuf *, void *, 
-		struct lowdown_node *);
-
-/* XXX: will be deprecated. */
-
-void	 lowdown_html_smrt(hbuf *, const char *, size_t);
-void	 lowdown_nroff_smrt(hbuf *, const char *, size_t);
+void 	 lowdown_tree_rndr(hbuf *, struct lowdown_metaq *,
+		void *, const struct lowdown_node *);
 
 __END_DECLS
 
